import attr
import cattr
from openpyxl.worksheet.worksheet import Worksheet
from typing import Dict

from . import Akses

MAPPING = {
    "K.P422_1RS": "rs",
    "K.P422_2bersalin": "bersalin",
    "K.P422_3Poliklinik": "poliklinik",
    "K.P422_4Puskesmas": "puskesmas",
    "K.P422_5pustu": "pustu",
    "K.P422_6Polindes": "polindes",
    "K.P422_7Poskesdes": "poskesdes",
    "K.P422_8Posyandu": "posyandu",
    "K.P422_9Apotik": "apotik",
    "K.P422_10Toko": "toko",
}

MAPPING_VALUE = {
    "rs": ("BM", "BN", "BO"),
    "bersalin": ("BP", "BQ", "BR"),
    "poliklinik": ("BS", "BT", "BU"),
    "puskesmas": ("BV", "BW", "BX"),
    "pustu": ("BY", "BZ", "CA"),
    "polindes": ("CB", "CC", "CD"),
    "poskesdes": ("CE", "CF", "CG"),
    "posyandu": ("CH", "CI", "CJ"),
    "apotik": ("CK", "CL", "CM"),
    "toko": ("CN", "CO", "CP"),
}


@attr.dataclass
class AksesFasilitasKesehatan:
    rs: Akses = attr.field(default=attr.Factory(Akses))
    bersalin: Akses = attr.field(default=attr.Factory(Akses))
    poliklinik: Akses = attr.field(default=attr.Factory(Akses))
    puskesmas: Akses = attr.field(default=attr.Factory(Akses))
    pustu: Akses = attr.field(default=attr.Factory(Akses))
    polindes: Akses = attr.field(default=attr.Factory(Akses))
    poskesdes: Akses = attr.field(default=attr.Factory(Akses))
    posyandu: Akses = attr.field(default=attr.Factory(Akses))
    apotik: Akses = attr.field(default=attr.Factory(Akses))
    toko: Akses = attr.field(default=attr.Factory(Akses))

    def todict(self) -> Dict[str, Dict[str, str]]:
        data: Dict[str, Dict[str, str]] = dict()
        for key, name in MAPPING.items():
            value: Akses = getattr(self, name)
            data[key] = cattr.unstructure(value)
        return data

    @staticmethod
    def make(ws: Worksheet, row: int) -> Dict[str, Dict[str, str]]:
        data: Dict[str, Dict[str, str]] = dict()
        for name, args in MAPPING_VALUE.items():
            data[name] = Akses.from_cols(ws, row, *args)
        return data

    def save(self, ws: Worksheet, row: int):
        for name, args in MAPPING_VALUE.items():
            akses: Akses = getattr(self, name)
            akses.save(ws, row, *args)



import attr
from openpyxl.worksheet.worksheet import Worksheet


@attr.dataclass
class Luas:
    lantai: str
    lahan: str

    @staticmethod
    def make(ws: Worksheet, row: int):
        return {"lantai": ws[f"L{row}"].value, "lahan": ws[f"M{row}"].value}



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "WNI": "1",
    "WNA": "2",
}


class Warganegara(Enum):
    WNI = "1"
    WNA = "2"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Genteng": "1",
    "Kayu/Jerami": "2",
    "Lainnya": "other",
}


class Atap(Enum):
    GENTENG = "1"
    KAYU_JERAMI = "2"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING, "other"))
    __str__ = make_dunder_str(MAPPING)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Laki-laki": "1",
    "Perempuan": "2",
}


class JenisKelamin(Enum):
    LAKI_LAKI = "1"
    PEREMPUAN = "2"

    from_str = classmethod(make_str_to_enum(MAPPING))
    __str__ = make_dunder_str(MAPPING)



import attr
from typing import Dict, List, Optional

from .enums import YaTidak


MAPPING = {
    "BLT DD": "blt_dd",
    "PKH": "pkh",
    "BST": "bst",
    "Banpres": "banpres",
    "UMKM": "umkm",
    "Bantuan Pekerja": "bantuan_pekerja",
    "Bantuan Pendidikan Anak": "bantuan_pendidikan_anak",
    "Lainnya": "lainnya",
}


@attr.dataclass
class BantuanPemerintah:
    blt_dd: YaTidak = YaTidak.TIDAK
    pkh: YaTidak = YaTidak.TIDAK
    bst: YaTidak = YaTidak.TIDAK
    banpres: YaTidak = YaTidak.TIDAK
    umkm: YaTidak = YaTidak.TIDAK
    bantuan_pekerja: YaTidak = YaTidak.TIDAK
    bantuan_pendidikan_anak: YaTidak = YaTidak.TIDAK
    lainnya: YaTidak = YaTidak.TIDAK

    def __str__(self):
        values: List[str] = list()
        for key, name in MAPPING.items():
            value = getattr(self, name)
            if value == YaTidak.YA:
                values.append(key)
        if not values:
            return ""
        return ", ".join(values)

    def todict(self) -> Dict[str, str]:
        return {
            "1": self.blt_dd.value,
            "2": self.pkh.value,
            "3": self.bst.value,
            "4": self.banpres.value,
            "5": self.umkm.value,
            "6": self.bantuan_pekerja.value,
            "7": self.bantuan_pendidikan_anak.value,
            "8": self.lainnya.value,
        }

    @classmethod
    def from_str(cls, val: Optional[str], t=None) -> "BantuanPemerintah":
        if not val:
            return cls()
        elif isinstance(val, dict):
            return cls.from_dict(val)
        data: Dict[str, YaTidak] = dict()
        for key, name in MAPPING.items():
            if key in val:
                data[name] = YaTidak.YA
            else:
                data[name] = YaTidak.TIDAK
        return cls(**data)

    @classmethod
    def from_dict(cls, val: Dict[str, str]) -> "BantuanPemerintah":
        data: Dict[str, YaTidak] = dict()
        for name, value in val.items():
            if value == "Ya":
                data[name] = YaTidak.YA
            else:
                data[name] = YaTidak.TIDAK
        return cls(**data)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Milik Sendiri": "1",
    "Milik Orang Lain": "2",
    "Tanah Negara": "3",
    "Lainnya": "other",
}


class StatusLahan(Enum):
    MILIK_SENDIRI = "1"
    MILIK_ORANG_LAIN = "2"
    TANAH_NEGARA = "3"
    LAINNYA = "other"

    def __str__(self):
        for key, value in MAPPING.items():
            if value == self.value:
                return key
        return ""

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



from uiautomator2 import Device, UiObject
from typing import Any, Dict, Optional

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to

from .fasilitas import get_data_fasilitas_kesehatan
from .penyakit import get_data_penyakit


__all__ = [
    "get_data_fasilitas_kesehatan",
    "get_data_penyakit",
]


def get_data_kesehatan(d: Device) -> Dict[str, Any]:
    menu_to(d, "KESEHATAN")
    data: Dict[str, Any] = dict()
    # d(text="KESEHATAN").click()
    data["penyakit_diderita"] = get_data_penyakit(d)
    data["fasilitas_kesehatan"] = get_data_fasilitas_kesehatan(d)
    jamsos_kesehatan = d_get_text(
        d, "com.kemendes.survey:id/cbJaminanKesehatanIndividu"
    )
    if jamsos_kesehatan == "Peserta":
        data["jamsos_kesehatan"] = "Ya"
    else:
        data["jamsos_kesehatan"] = "Tidak"
    data["setahun_melahirkan"] = d_get_text(d, "com.kemendes.survey:id/cbMelahirkan")
    d(className="android.widget.ScrollView").fling.vert.backward()
    return data
